@@ -728,7 +728,8 @@ class PatchTransform3D:
         self.o(y)
         return self
 
-    def apply(self,patch_shape,stride=None,transform_name = None,random_patches=False,n_random_patches=4000):
+    def apply(self,patch_shape,stride=None,vol_frac_th = 0.05,transform_name = None,random_patches=False,
+              n_random_patches=4000):
         """
         Apply the operation to the trace. The function 'transform_function' specified during the operation
         initialization is applied to various patches of the input data. The application can be done:
@@ -741,6 +742,9 @@ class PatchTransform3D:
         :param patch_shape: (tuple[int]) shape of the input patch used in this operation.
         :param stride: (None or tuple[int]) optional, stride along each dimension (used when the transformation is
                        taken along a ZYX-grid).
+        :param vol_frac_th: (float between 0 and 1) fraction of the patch that need to be occupied by 1s in order to
+                            consider a patch suitable for the transformation (i.e. when there are too many zeros the
+                            patch is disregarded).
         :param transform_name: (str) optional, name given to the transformation output (one of the two keys of the
                                output dictionary of this operation)
         :param random_patches: (bool) optional, if True the patch transform is applied in random points of the input
@@ -754,6 +758,7 @@ class PatchTransform3D:
             self.patch_shape = patch_shape
             self.stride = stride
             self.transform_name = transform_name
+            self.vol_frac_th = vol_frac_th
             self.random_patches = random_patches
             self.n_random_patches = n_random_patches
 
@@ -857,7 +862,7 @@ class PatchTransform3D:
                 xmin = int(x*self.patch_shape[2])
                 xmax = xmin+self.patch_shape[2]
                 patch = in_db[zmin:zmax,ymin:ymax,xmin:xmax]
-                if np.sum(patch) > 0:
+                if np.sum(patch) > self.vol_frac_th*np.prod(self.patch_shape):
 
                     # compute transformed patch and coordinates in the output patch shape
                     tmp_db[m,:,:,:] = self._patch_transform(patch)
@@ -949,7 +954,7 @@ class PatchTransform3D:
                 xmin = np.random.randint(self.patch_shape[2],self.data_shape[2]-self.patch_shape[2])
                 xmax = xmin+self.patch_shape[2]
                 patch = in_db[zmin:zmax,ymin:ymax,xmin:xmax]
-                if np.sum(patch) > 0:
+                if np.sum(patch) > self.vol_frac_th*np.prod(self.patch_shape):
 
                     # compute transformed patch
                     db[len(Zs),:,:,:] = self._patch_transform(patch)
@@ -1146,8 +1151,8 @@ class PatchDiscreteFourierTransform3D:
         self.o(y)
         return self
 
-    def apply(self,patch_shape,representation = 'module_phase',random_patches=False,n_random_patches=4000,
-              use_periodic_smooth_decomposition=True):
+    def apply(self,patch_shape,representation = 'module_phase',vol_frac_th=0.05,random_patches=False,
+              n_random_patches=4000,use_periodic_smooth_decomposition=True):
         """
         Apply the operation to the trace. The function 'transform_function' specified during the operation
         initialization is applied to various patches of the input data. The application can be done:
@@ -1160,6 +1165,9 @@ class PatchDiscreteFourierTransform3D:
         :param patch_shape: (tuple[int]) shape of the patch used in this operation.
         :param representation: (str) optional, it can be 'module_phase' or 'real_imaginary' depending if one want to
                                save the module and phase of the DFT or the real and imaginary part of the DFT.
+        :param vol_frac_th: (float between 0 and 1) fraction of the patch that need to be occupied by 1s in order to
+                            consider a patch suitable for the transformation (i.e. when there are too many zeros the
+                            patch is disregarded).
         :param random_patches: (bool) optional, if True the patch transform is applied in random points of the input
                                dataset, otherwise an ZYX-grid is used.
         :param n_random_patches: (int) optional, number of random patches taken (this variable is considered only when
@@ -1172,6 +1180,7 @@ class PatchDiscreteFourierTransform3D:
 
             self.patch_shape = patch_shape
             self.representation = representation
+            self.vol_frac_th = vol_frac_th
             self.random_patches = random_patches
             self.n_random_patches = n_random_patches
             self.use_periodic_smooth_decomposition = use_periodic_smooth_decomposition
@@ -1284,7 +1293,7 @@ class PatchDiscreteFourierTransform3D:
                 xmin = int(x*self.patch_shape[2])
                 xmax = xmin+self.patch_shape[2]
                 patch = in_db[zmin:zmax,ymin:ymax,xmin:xmax]
-                if np.sum(patch) > 0:
+                if np.sum(patch) > self.vol_frac_th*np.prod(self.patch_shape):
 
                     # compute transformed patch
                     t_patch = self._patch_transform(patch)
@@ -1401,7 +1410,7 @@ class PatchDiscreteFourierTransform3D:
                 xmin = np.random.randint(self.patch_shape[2],self.data_shape[2]-self.patch_shape[2])
                 xmax = xmin+self.patch_shape[2]
                 patch = in_db[zmin:zmax,ymin:ymax,xmin:xmax]
-                if np.sum(patch) > 0:
+                if np.sum(patch) > self.vol_frac_th*np.prod(self.patch_shape):
 
                     # compute transformed patch
                     t_patch = self._patch_transform(patch)
